////////////////////////////////////////////////////////////////////
// Sudko solver. 
// Written by Vladeta Stojanovic (vladeta_stojanovic@yahoo.co.uk)
//
// Version: 230412_01
////////////////////////////////////////////////////////////////////

package  
{
	//Imports
	import com.adobe.serialization.json.JSONDecoder;
	import flash.events.*;
	import flash.net.FileFilter;
	import flash.net.FileReference;
	import flash.text.TextFormat;
    import flash.utils.*;
    import flash.display.*;
	import flash.text.TextField;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import org.flashdevelop.utils.FlashConnect;
	import mx.core.UIComponent;
	
	public class Main extends Sprite
	{	
		//class variables
		private var SCREEN_WIDTH : int = 512;
		private var SCREEN_HEIGHT : int = 512;
		private var parsedData : Array = new Array(0);
		private var timer : Timer;
		private var solved : Boolean = false;
		
		//program main
		public function Main() 
		{
			//Create & init text indfo
			trace("Initializing Solver");
			
			timer = new Timer(10);
			
			Init("test.txt");
		}
		
		//Init everything here
		public function Init(inputFile : String) : void 
		{
			trace ("loading test file...");
			
			 var loader:URLLoader = new URLLoader();
			 var request:URLRequest = new URLRequest(inputFile);
			 loader.addEventListener(Event.COMPLETE, onLoadComplete);
			 loader.load(request);
			
			trace ("file loaded...");
		}
		
		//used for cleaning the "[" & "]" formt he JSON array string structure
		private function searchReplace(block:String, find:String, replace:String):String
		{
			return block.split(find).join(replace);
		} 
		
		///////////////////////////////////////////////
		//Sudoku JSON parsing, processing, & output
		//////////////////////////////////////////////
		public function onLoadComplete(e : Event) : void 
		{
			//read in the data into one big array & clean the array some more
			e.target.data = searchReplace(String(e.target.data), "[", " ");
			e.target.data = searchReplace(String(e.target.data), "]", " ");
			parsedData = String(e.target.data).split(',');

			var linesNum:int = parsedData.length;

			//grid preview varaibles
			var gridInfo : TextField = new TextField();
			gridInfo.width = 500;
			gridInfo.height = 500;
			gridInfo.x = 25;
			gridInfo.y = 25;
			gridInfo.text = "Input Grid: \n\n";
			addChild(gridInfo);
		   
			for (var i : int = 0 ; i < linesNum; i++)
			{
			  //convert all string data into unsigned ints
			  parsedData[i] = uint(parsedData[i]);
			  trace('line ' + i + ': ' + parsedData[i]);
			}
			
			trace ("data parsed");

			//Display the preview grid (fix this....)
			for (var j : int = 0; j <= 81; j++) 
			{
				if (j < 9) gridInfo.appendText(parsedData[j]);
				if (j == 9) gridInfo.appendText("\n");
				if (j >= 9 && j < 18) gridInfo.appendText(parsedData[j]);
				if (j == 18) gridInfo.appendText("\n");
				if (j >= 18 && j < 27) gridInfo.appendText(parsedData[j]);
				if (j == 27) gridInfo.appendText("\n");
				if (j >= 27 && j < 36) gridInfo.appendText(parsedData[j]);
				if (j == 36) gridInfo.appendText("\n");
				if (j >= 36 && j < 45) gridInfo.appendText(parsedData[j]);
				if (j == 45) gridInfo.appendText("\n");
				if (j >= 45 && j < 54) gridInfo.appendText(parsedData[j]);
				if (j == 54) gridInfo.appendText("\n");
				if (j >= 54 && j < 63) gridInfo.appendText(parsedData[j])
				if (j == 63) gridInfo.appendText("\n");
				if (j >= 63 && j < 72) gridInfo.appendText(parsedData[j])
				if (j == 72) gridInfo.appendText("\n");
				if (j >= 72 && j < 81) gridInfo.appendText(parsedData[j])
			}
			
			
			//call sudoku solver here
			var i : int = 0;
			
			while (SolveSudoku(i) == false)
			{
				if (i <= 81)
				{
					++i;
				}
				trace ("working...");
			}
			
		
			//display the new solved grid
			var solvedInfo : TextField = new TextField();
			solvedInfo.width = 500;
			solvedInfo.height = 500;
			solvedInfo.x = 325;
			solvedInfo.y = 25;
			solvedInfo.text = "Solved Grid: \n\n";
			addChild(solvedInfo);
			
			for (var k : int = 0; k <= 81; k++) 
			{
				if (k < 9) solvedInfo.appendText(parsedData[k]);
				if (k == 9) solvedInfo.appendText("\n");
				if (k >= 9 && k < 18) solvedInfo.appendText(parsedData[k]);
				if (k == 18) solvedInfo.appendText("\n");
				if (k >= 18 && k < 27) solvedInfo.appendText(parsedData[k]);
				if (k == 27) solvedInfo.appendText("\n");
				if (k >= 27 && k < 36) solvedInfo.appendText(parsedData[k]);
				if (k == 36) solvedInfo.appendText("\n");
				if (k >= 36 && k < 45) solvedInfo.appendText(parsedData[k]);
				if (k == 45) solvedInfo.appendText("\n");
				if (k >= 45 && k < 54) solvedInfo.appendText(parsedData[k]);
				if (k == 54) solvedInfo.appendText("\n");
				if (k >= 54 && k < 63) solvedInfo.appendText(parsedData[k])
				if (k == 63) solvedInfo.appendText("\n");
				if (k >= 63 && k < 72) solvedInfo.appendText(parsedData[k])
				if (k == 72) solvedInfo.appendText("\n");
				if (k >= 72 && k < 81) solvedInfo.appendText(parsedData[k])
			}
			
			//output data
			OutputResults();
			
			trace ("program finished");
 
		}
		
		
		//Based on the backtracking algorithm.
		//References: http://en.algoritmy.net/article/42521/Sudoku
		//			  http://en.wikipedia.org/wiki/Back_tracking
		//			  http://en.wikipedia.org/wiki/Sudoku_algorithms
		//			  http://www.codeproject.com/Articles/11983/Sudoku-Solver-and-Generator		  
		private function SolveSudoku(index : int) : Boolean
		{
			var row:Number = Math.floor(index / 9);
			var col:Number = index - (row * 9);

			if (index < 81) 
			{
				if (parsedData[row * 9 + col] != 0) 
				{
					return SolveSudoku(index + 1);
				}	 
				else 
				{
					for (var i = 1; i <= 9; i++) 
					{
						if (IsConsistent(row, col, i)) 
						{
							parsedData[row * 9 + col] = i;
							if (SolveSudoku(index + 1)) 
							{
								return true;
							}
							
							parsedData[row * 9 + col] = 0;
						} 
						else if (i == 9) 
						{
							return false;
						}
					}
				}
				return false;
			} 
			else 
			{
				return true;
			}
		}
		
		
		private function IsConsistent(row : int, col : int, value : int) : Boolean
		{
			  for (var i = 0; i < 9; i++) 
			  {
				if ((value == parsedData[row * 9 + row])) 
				{	
					return false;
				}
				if ((value == parsedData[col * 9 + col])) 
				{
					return false;
				}
				
			  }

			  var bcol : int = col - (col % 3);
			  var brow : int = row - (row % 3);
			  
			  for (var i : int = 0; i < 3; i++) 
			  {
				for (var j : int = 0; j < 3; j++) 
				{
					if ((value == parsedData[(brow + i )* 9 + (bcol + j)])) 
					{
						return false;
					}
				}
			  }
			  return true;
		}
		
		private function OutputResults() : void
		{
			var fileRef:FileReference = new FileReference();
			var outInfo : TextField = new TextField();
			outInfo.width = 500;
			outInfo.height = 500;
			outInfo.x = 325;
			outInfo.y = 25;
			
			for (var k : int = 0; k <= 81; k++) 
			{
				if (k == 0) outInfo.appendText("[");
				if (k < 9) outInfo.appendText(parsedData[k] + ",");
				if (k == 9) outInfo.appendText("\n");
				if (k >= 9 && k < 18) outInfo.appendText(parsedData[k] + ",");
				if (k == 18) outInfo.appendText("\n");
				if (k >= 18 && k < 27) outInfo.appendText(parsedData[k] + ",");
				if (k == 27) outInfo.appendText("\n");
				if (k >= 27 && k < 36) outInfo.appendText(parsedData[k] + ",");
				if (k == 36) outInfo.appendText("\n");
				if (k >= 36 && k < 45) outInfo.appendText(parsedData[k] + ",");
				if (k == 45) outInfo.appendText("\n");
				if (k >= 45 && k < 54) outInfo.appendText(parsedData[k] + ",");
				if (k == 54) outInfo.appendText("\n");
				if (k >= 54 && k < 63) outInfo.appendText(parsedData[k] + ",")
				if (k == 63) outInfo.appendText("\n");
				if (k >= 63 && k < 72) outInfo.appendText(parsedData[k] + ",")
				if (k == 72) outInfo.appendText("\n");
				if (k >= 72 && k < 80) outInfo.appendText(parsedData[k] + ",")
				if (k == 80)  outInfo.appendText(parsedData[k] + "]");
			}
			
			fileRef.cancel();
			fileRef.save(outInfo.text, "testOut.txt");
		}
		
	}
	
}